#ifndef __EXPORTER_H
#define __EXPORTER_H

#include <string>
#include <vector>
using namespace std;

#include "Material.h"

class KdTree;
class SceneGraphNode;
class Ray;
class AxisAlignedBox3;
class Intersectable;

class Exporter
{
protected:
  string mFilename;
  bool wireframe;
  bool mUseForcedMaterial;
  Material mForcedMaterial;

public:
  
  Exporter(const string filename):mFilename(filename),
				  wireframe(false),
				  mUseForcedMaterial(false)
  {
  }
  virtual ~Exporter() {}
  
  virtual bool ExportScene(SceneGraphNode *node) = 0;

  virtual bool
  ExportBox(const AxisAlignedBox3 &box) = 0;

  virtual bool
  ExportKdTree(const KdTree &tree) = 0;

  virtual bool
  ExportRays(const vector<Ray> &rays,
	     const float length=1000,
	     const RgbColor &color = RgbColor(1,1,1)
	     ) = 0;
  

  virtual void
  ExportIntersectable(Intersectable *object) = 0;

  void SetWireframe() { wireframe = true; }
  void SetFilled() { wireframe = false; }

  void SetForcedMaterial(const Material &m) {
    mForcedMaterial = m;
    mUseForcedMaterial = true;
  }
  void ResetForcedMaterial() {
    mUseForcedMaterial = false;
  }
    
  static Exporter *
  GetExporter(const string filename);

  
};
 


#endif
